#ifdef	__TURBOC__
#include <alloc.h>
#else
#include <malloc.h>
#endif
#include "stdio.h"
#include "matrix.h"

/*-----------------------------------------------------------------------------
*	desc:	matrix mathematics - object creation
*	by:	ko shu pui, patrick
*	date:	24 nov 91 v0.1
*	revi:	14 may 92 v0.2
*		21 may 92 v0.3
*		31 ott 96 v0.4 By Diego Bonata
*	ref:
*       [1] Mary L.Boas, "Mathematical Methods in the Physical Sciene,"
*	John Wiley & Sons, 2nd Ed., 1983. Chap 3.
*
*	[2] Kendall E.Atkinson, "An Introduction to Numberical Analysis,"
*	John Wiley & Sons, 1978.
*
*-----------------------------------------------------------------------------*/
MATRIX	_mat_creat(int row, int col )
{
MATBODY	*mat;
int 	i, j;

 if ((mat = (MATBODY *)malloc( sizeof(MATHEAD) + sizeof(double *) * row)) == NULL)
		return (mat_error( MAT_MALLOC ));
 for (i=0; i<row; i++)	{
     if ((*((double **)(&mat->matrix) + i) = (double *)malloc(sizeof(double) * col)) == NULL)
		return (mat_error( MAT_MALLOC ));
 }
 mat->head.row = row;
 mat->head.col = col;

return (&(mat->matrix));
}

/*-----------------------------------------------------------------------------
*	funct:	mat_creat
*	desct:	create a matrix
*	given:  row, col = dimension, type = which kind of matrix
*	retrn:	allocated matrix (use mat_free() to free memory)
*-----------------------------------------------------------------------------*/
MATRIX	mat_creat( row, col, type )
int row, col, type;
{
MATRIX	A;

 if ((A =_mat_creat( row, col )) != NULL) return (mat_fill(A, type));
 else                            	  return (NULL);
}

/*-----------------------------------------------------------------------------
*	funct:	mat_fill
*	desct:	form a special matrix
*	given:  A = matrix, type = which kind of matrix
*	retrn:	A
*-----------------------------------------------------------------------------*/
MATRIX mat_fill(MATRIX A,int type)
{
int	i, j;

 switch (type)		{
     case UNDEFINED:			break;
     case ZERO_MATRIX:
     case UNIT_MATRIX:	for (i=0; i<MatRow(A); i++)
			for (j=0; j<MatCol(A); j++)	{
				if (type == UNIT_MATRIX) {
					if (i==j) A[i][j] = 1.0;
					continue;
				}
				A[i][j] = 0.0;
			}
			break;
 }
return (A);
}

/*-----------------------------------------------------------------------------
*	funct:	mat_free
*	desct:	free an allocated matrix
*	given:  A = matrix
*	retrn:	nothing <actually 0 = NULL A passed, 1 = normal exit>
*-----------------------------------------------------------------------------*/
int mat_free(MATRIX A)
{
int i;

 if (A == NULL)		     return (0);
 for (i=0; i<MatRow(A); i++) free( A[i] );
 free( Mathead(A) );

return (1);
}

/*-----------------------------------------------------------------------------
*	funct:	mat_copy
*	desct:	duplicate a matrix
*	given:	A = matrice to duplicated
*	retrn:	C = A
*	comen:
*-----------------------------------------------------------------------------*/
MATRIX mat_copy(MATRIX A)
{
int	i, j;
MATRIX	C;

 if ((C = mat_creat( MatRow(A), MatCol(A), UNDEFINED )) == NULL) return (NULL);
 for (i=0; i<MatRow(A); i++)
	for (j=0; j<MatCol(A); j++)	C[i][j] = A[i][j];

return (C);
}

MATRIX mat_colcopy1(MATRIX A,MATRIX B,int cola,int colb)
{
int	i, n;

 n = MatRow(A);
 for (i=0; i<n; i++)	A[i][cola] = B[i][colb];

return (A);
}

int fgetmat(MATRIX A,FILE *fp)
{
int 	i, j, k=0;

 for (i=0; i<MatRow(A); i++)
	for (j=0; j<MatCol(A); j++)
		{
//	to avoid a bug in TC

#ifdef	__TURBOC__
		{
		double	temp;
		k += fscanf( fp, "%lf", &temp );
		A[i][j] = temp;
		}
#else
		k += fscanf( fp, "%lf", &A[i][j] );
#endif

 }

return (k);
}

/*-----------------------------------------------------------------------------
*	file:	materr.c
*	desc:	matrix error handler
*	by:	ko shu pui, patrick
*	date:	24 nov 91 v0.1
*	revi:   31 ott 96 v0.2 By Diego Bonata
*	ref:
*	[1] Mary L.Boas, "Mathematical Methods in the Physical Sciene,"
*	John Wiley & Sons, 2nd Ed., 1983. Chap 3.
*
*	[2] Kendall E.Atkinson, "An Introduction to Numberical Analysis,"
*	John Wiley & Sons, 1978.
*-----------------------------------------------------------------------------*/
MATRIX mat_error(int errno)
{
 switch( errno )	{
     case MAT_MALLOC:	 fprintf(stderr, "mat: malloc error\n" );
			 break;
     case MAT_FNOTOPEN:	 fprintf(stderr, "mat: fileopen error\n" );
			 break;
     case MAT_FNOTGETMAT:fprintf(stderr, "fgetmat: matrix read error\n");
			 break;
 }
return (NULL);
}

/*-----------------------------------------------------------------------------
*	desc:	solve linear equations
*	by:	ko shu pui, patrick
*	date:	24 nov 91 v0.1
*	revi:	14 may 92 v0.2
*               31 ott 96 v0.3 By Diego Bonata
*	ref:
*       [1] Mary L.Boas, "Mathematical Methods in the Physical Sciene,"
*	John Wiley & Sons, 2nd Ed., 1983. Chap 3.
*
*	[2] Kendall E.Atkinson, "An Introduction to Numberical Analysis,"
*	John Wiley & Sons, 1978.
*
*-----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
*	funct:	mat_lu
*	desct:	in-place LU decomposition with partial pivoting
*	given:	!! A = square matrix (n x n) !ATTENTION! see commen
*		P = permutation vector (n x 1)
*	retrn:	number of permutation performed
*		-1 means suspected singular matrix
*	comen:	A will be overwritten to be a LU-composite matrix
*
*	note:	the LU decomposed may NOT be equal to the LU of
*		the orignal matrix a. But equal to the LU of the
*		rows interchanged matrix.
*-----------------------------------------------------------------------------*/
int mat_lu(MATRIX A,MATRIX P)
{
int	i, j, k, n;
int	maxi, tmp;
double	c, c1;
int	p;

 n = MatCol(A);
 for (p=0,i=0; i<n; i++)		P[i][0] = i;
 for (k=0; k<n; k++)     {
     // --- partial pivoting ---
     for (i=k, maxi=k, c=0.0; i<n; i++)	{
		c1 = fabs( A[P[i][0]][k] );
		if (c1 > c)	{
			c = c1;
			maxi = i;
		}
     }
     //	row exchange, update permutation vector
     if (k != maxi)	{
		p++;
		tmp = P[k][0];
		P[k][0] = P[maxi][0];
		P[maxi][0] = tmp;
     }
     //	suspected singular matrix
     if ( A[P[k][0]][k] == 0.0 )	return (-1);
     for (i=k+1; i<n; i++)		{
	// --- calculate m(i,j) ---
	A[P[i][0]][k] = A[P[i][0]][k] / A[P[k][0]][k];
	// --- elimination ---
	for (j=k+1; j<n; j++) A[P[i][0]][j] -= A[P[i][0]][k] * A[P[k][0]][j];
     }
 }
return (p);
}

/*-----------------------------------------------------------------------------
*	funct:	mat_backsubs1
*	desct:	back substitution
*	given:	A = square matrix A (LU composite)
*		!! B = column matrix B (attention!, see comen)
*		!! X = place to put the result of X
*		P = Permutation vector (after calling mat_lu)
*		xcol = column of x to put the result
*	retrn:	column matrix X (of AX = B)
*	comen:	B will be overwritten
*-----------------------------------------------------------------------------*/
MATRIX mat_backsubs1(MATRIX A,MATRIX B,MATRIX X,MATRIX P,int xcol)
{
int	i, j, k, n;
double	sum;

 n = MatCol(A);
 for (k=0; k<n; k++)	{
    for (i=k+1; i<n; i++)
		B[P[i][0]][0] -= A[P[i][0]][k] * B[P[k][0]][0];
 }
 X[n-1][xcol] = B[P[n-1][0]][0] / A[P[n-1][0]][n-1];
 for (k=n-2; k>=0; k--)		{
     sum = 0.0;
     for (j=k+1; j<n; j++) sum += A[P[k][0]][j] * X[j][xcol];
     X[k][xcol] = (B[P[k][0]][0] - sum) / A[P[k][0]][k];
 }

return (X);
}

/*-----------------------------------------------------------------------------
*	funct:	mat_lsolve
*	desct:	solve linear equations
*	given:	a = square matrix A
*		b = column matrix B
*	retrn:	column matrix X (of AX = B)
*-----------------------------------------------------------------------------*/
MATRIX mat_lsolve(MATRIX a,MATRIX b)
{
MATRIX	A, B, X, P;
int	i, n;
double	temp;

 n = MatCol(a);
 A = mat_copy(a);
 B = mat_copy(b);
 X = mat_creat(n, 1, ZERO_MATRIX);
 P = mat_creat(n, 1, UNDEFINED);

 mat_lu( A, P );
 mat_backsubs1( A, B, X, P, 0 );

 mat_free(A);
 mat_free(B);
 mat_free(P);
return (X);
}

/*-----------------------------------------------------------------------------
*	funct:	mat_tran
*	desct:	transpose of a matrix
*	given:	A = matrix A to be transposed
*	retrn:	allocated matrix for A^t
*	comen:
*---------------------------------------------------------------------------*/
MATRIX mat_tran( A )
MATRIX A;
{
int	i, j;
MATRIX	At;

 if ((At=mat_creat(MatCol(A),MatRow(A),UNDEFINED))==NULL) return (NULL);
 /*	Transposing ...	*/
 for (i=0; i<MatCol(A); i++)
 for (j=0; j<MatRow(A); j++) At[i][j] = A[j][i];

return (At);
}
