#include "math.h"
#include "stdio.h"
/*-----------------------------------------------------------------------------
*	file:	matrix.h
*	desc:	matrix mathematics header file
*	by:	ko shu pui, patrick
*	date:	24 nov 91	v0.1b
*	revi:   Diego Bonata
*	ref:
*       [1] Mary L.Boas, "Mathematical Methods in the Physical Sciene,"
*	John Wiley & Sons, 2nd Ed., 1983. Chap 3.
*
*-----------------------------------------------------------------------------*/


//-----------------------------------------------------------------------------
// Struttura interna delle matrici
//-----------------------------------------------------------------------------

typedef struct {
	int	row;
	int	col;
	}	MATHEAD;

typedef struct {
	MATHEAD	head;
	double	*matrix;
	}	MATBODY;

typedef	double	**MATRIX;

#define	Mathead(a)	((MATHEAD *)((MATHEAD *)(a) - 1))
#define MatRow(a)	(Mathead(a)->row)
#define	MatCol(a)	(Mathead(a)->col)

//----------------------------------------------------------------------------
// Definizione interna dei tipi di errore
//----------------------------------------------------------------------------

#define	MAT_MALLOC	1
#define MAT_FNOTOPEN	2
#define	MAT_FNOTGETMAT	3

//----------------------------------------------------------------------------
//	Tipi di matrice
//----------------------------------------------------------------------------

#define UNDEFINED	-1
#define ZERO_MATRIX	0
#define	UNIT_MATRIX	1

/* prototypes of matrix package */
#ifdef __TURBOC__

MATRIX mat_error	(int);
MATRIX _mat_creat	(int, int);
MATRIX mat_creat	(int, int, int);
MATRIX mat_fill		(MATRIX, int);
int mat_free		(MATRIX);
MATRIX mat_copy		(MATRIX);
MATRIX mat_colcopy1	(MATRIX, MATRIX, int, int);
int fgetmat		(MATRIX, FILE *);
MATRIX mat_tran		(MATRIX);
int mat_lu		(MATRIX, MATRIX);
MATRIX mat_backsubs1	(MATRIX, MATRIX, MATRIX, MATRIX, int);
MATRIX mat_lsolve	(MATRIX, MATRIX);
#else

MATRIX mat_error	();
MATRIX _mat_creat	();
MATRIX mat_creat	();
MATRIX mat_fill		();
int mat_free		();
MATRIX mat_copy		();
MATRIX mat_colcopy1	();
int fgetmat		();
MATRIX mat_tran		();
int mat_lu		();
MATRIX mat_backsubs1	();
MATRIX mat_lsolve	();
#endif

